package willmatheson.com.example.quizbuilder;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Random;
import java.util.Collections;
import java.util.Vector;

import android.os.Bundle;
import android.app.Activity;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.view.Menu;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

public class QuizActivity extends Activity {
	
	//interactive objects
	TextView quizFor;
	TextView textViewPH;
	Button quitbutton;
	Button button1;
	Button button2;
	Button button3;
	Button button4;
	//definitions storage
	private ArrayList<String> quizDefinitionList;
	//terms storage
	private HashMap<Integer, String> quizTermMap;
    //helpers to drive the program
	private Vector<Integer> questions;
	private int questionCount = 0;
	private int rightChoice = 0;
	
	
    //onCreate: this loads when the activity_quiz form loads
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //make it white-on-black
        setTheme(android.R.style.Theme_Holo);
        //lock rotation (redoing onCreate causes problems)
        this.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        setContentView(R.layout.activity_quiz);
        
        //link to quizFor TextView on layout (gets user's name from intent)
        TextView quizFor = (TextView) findViewById(R.id.textViewQuizFor);
        
        //link to buttons on layout
        Button quitButton = (Button) findViewById(R.id.quitButton);
    	Button button1 = (Button) findViewById(R.id.button1);
    	Button button2 = (Button) findViewById(R.id.button2);
    	Button button3 = (Button) findViewById(R.id.button3);
    	Button button4 = (Button) findViewById(R.id.button4);
        
        //set action listener for quit button
        quitButton.setOnClickListener(new View.OnClickListener()
        {
        	public void onClick(View v)
        	{
        		quitMe();
        	}
        });   
        
        //set action listeners for answer buttons
        button1.setOnClickListener(new View.OnClickListener()
        {
        	public void onClick(View v)
        	{
        		button1();
        	}
        });
        button2.setOnClickListener(new View.OnClickListener()
        {
        	public void onClick(View v)
        	{
        		button2();
        	}
        });
        button3.setOnClickListener(new View.OnClickListener()
        {
        	public void onClick(View v)
        	{
        		button3();
        	}
        });
        button4.setOnClickListener(new View.OnClickListener()
        {
        	public void onClick(View v)
        	{
        		button4();
        	}
        });
        
        //bring in intent with user name
        String userName = "";
        Bundle extras = getIntent().getExtras();
        if (extras!=null){
        	userName= extras.getString("whoami");
        }
        if (userName.equals("")) userName = getString(R.string.defaultUser);
        
        //display that the quiz is for who it's for
        quizFor.setText(userName);
        
        //get array of definitions from strings.xml
        String[] definitions = getResources().getStringArray(R.array.definitionsList);
            
        //put them in the definition ArrayList
        quizDefinitionList = new ArrayList<String>();
        Collections.addAll(quizDefinitionList,definitions);
            
        //get array of terms from strings.xml
        String[] terms = getResources().getStringArray(R.array.termsList);
            
        //put them in the terms HashMap
        quizTermMap = new HashMap<Integer,String>();
        for (int i=0; i<terms.length; i++)
        {
        quizTermMap.put(i, terms[i]);
        }
        
        //make vector of ints from 0-n and use them to order definitions
        questions = new Vector<Integer>(quizDefinitionList.size());
        for (int i=0; i<quizDefinitionList.size(); i++){
              questions.add(Integer.valueOf(i));
        }
        
        //shuffle the question order
        Collections.shuffle(questions);
        
        //ask first question
        askQuestion(questions.get(questionCount));



    }
    
    //auto-generated
    //Initializes the contents of the Activity's standard options menu
    //but we don't have any
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.activity_quiz, menu);
        return true;
    }
    
    //asks question by question # (refers to the respective
    //indicies in the terms and definitions collections)
    public void askQuestion(int questionNo){
    	
    	//link to definition display area on layout
    	TextView textViewPH = (TextView) findViewById(R.id.textViewPH);
    	
    	//link to buttons on layout (to change their text)
    	Button button1 = (Button) findViewById(R.id.button1);
    	Button button2 = (Button) findViewById(R.id.button2);
    	Button button3 = (Button) findViewById(R.id.button3);
    	Button button4 = (Button) findViewById(R.id.button4);
    	
    	//put the definition on the display
    	textViewPH.setText(quizDefinitionList.get(questionNo));
    	
    	//generate keys for three nonsense terms
    	//prevents seeing the correct answer twice
    	//prevents seeing more than 1 of any incorrect answer
    	//(using arrayOccurrenceCount)
    	Random rand = new Random();
    	boolean okay; //if not okay, generate another random #
    	int randTerm = 0;
    	int randTerms[] = new int[3];
    	for (int i=0; i<randTerms.length; i++){
    		randTerms[i] = -1;
    		okay = false;
    	while (!okay){
    		randTerm = rand.nextInt(quizDefinitionList.size());
    		if ((randTerm != questionNo) && (arrayOccurrenceCount(randTerms,randTerm) == 0))
    		{
    			randTerms[i] = randTerm;
    			okay = true;
    		}
    	}}
    	
        //determine button that will get correct choice
    	rightChoice = rand.nextInt(4) + 1;
    	
    	//put the correct term on that random button
    	//populate others with nonsense terms
    	switch (rightChoice) {
    	  case 1: 
    	    button1.setText(quizTermMap.get(questionNo));
    	    button2.setText(quizTermMap.get(randTerms[0]));
    	    button3.setText(quizTermMap.get(randTerms[1]));
    	    button4.setText(quizTermMap.get(randTerms[2]));
    	    break;
    	  case 2: 
    		button2.setText(quizTermMap.get(questionNo));
    	    button1.setText(quizTermMap.get(randTerms[0]));
    	    button3.setText(quizTermMap.get(randTerms[1]));
    	    button4.setText(quizTermMap.get(randTerms[2]));
    	    break;
    	  case 3: 
    		button3.setText(quizTermMap.get(questionNo));
    	    button1.setText(quizTermMap.get(randTerms[0]));
    	    button2.setText(quizTermMap.get(randTerms[1]));
    	    button4.setText(quizTermMap.get(randTerms[2]));
    	    break;
    	  case 4: 
    		button4.setText(quizTermMap.get(questionNo));
    	    button1.setText(quizTermMap.get(randTerms[0]));
    	    button2.setText(quizTermMap.get(randTerms[1]));
    	    button3.setText(quizTermMap.get(randTerms[2]));
    	    break;
    	  default: 
    	    showMessage(getString(R.string.error));
    	}
    }
    
    //button 1 function
    public void button1(){
    	//if correct response, say so
    	if (rightChoice == 1){
    		showMessage(getString(R.string.correct));
    		//if there are more questions, ask next question
            if (questionCount < quizDefinitionList.size()-1){
    		questionCount += 1;
            askQuestion(questions.get(questionCount));}
            //else, quit
            else{ 
            	showMessage(getString(R.string.quizdone));
            	finish();
            }
    	}
    	//if incorrect, say so
    	else showMessage(getString(R.string.incorrect));
    }
    
    //button 2 function
    public void button2(){
    	//if correct response, say so
    	if (rightChoice == 2){
    		showMessage(getString(R.string.correct));
    		//if there are more questions, ask next question
            if (questionCount < quizDefinitionList.size()-1){
    		questionCount += 1;
            askQuestion(questions.get(questionCount));}
            //else, quit
            else{ 
            	showMessage(getString(R.string.quizdone));
            	finish();
            }
    	}
    	//if incorrect, say so
    	else showMessage(getString(R.string.incorrect));
    	
    }

    //button 3 function
    public void button3(){
    	//if correct response, say so
    	if (rightChoice == 3){
    		showMessage(getString(R.string.correct));
    		//if there are more questions, ask next question
            if (questionCount < quizDefinitionList.size()-1){
    		questionCount += 1;
            askQuestion(questions.get(questionCount));}
            //else, quit
            else{ 
            	showMessage(getString(R.string.quizdone));
            	finish();
            }
    	}
    	//if incorrect, say so
    	else showMessage(getString(R.string.incorrect));
    	
    }

    //button 4 function
    public void button4(){
    	//if correct response, say so
    	if (rightChoice == 4){
    		showMessage(getString(R.string.correct));
    		//if there are more questions, ask next question
            if (questionCount < quizDefinitionList.size()-1){
    		questionCount += 1;
            askQuestion(questions.get(questionCount));}
            //else, quit
            else{ 
            	showMessage(getString(R.string.quizdone));
            	finish();
            }
    	}
    	//if incorrect, say so
    	else showMessage(getString(R.string.incorrect));
    	
    }
    
    //quit button function
    public void quitMe(){
    	finish();
    }
    
    //shows the supplied string as a toast message
    //http://developer.android.com/guide/topics/ui/notifiers/toasts.html
    public void showMessage(String message){
    	Context context = getApplicationContext();
    	int duration = Toast.LENGTH_SHORT;
    	Toast toast = Toast.makeText(context, message, duration);
    	toast.show();
    }
    
    //takes an array of int and an int to check for
    //returns number of occurences of the int in that array
    //http://stackoverflow.com/q/11053607
    public int arrayOccurrenceCount(int[] array, int checkFor) {
        int occurrences = 0;
        for (int i = 0; i < array.length; i++) {
            if (array[i] == checkFor) {
                occurrences++;
            }
        }
        return occurrences;
    }
}
